# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['opsrampcli']

package_data = \
{'': ['*']}

install_requires = \
['DateTime>=4.4,<5.0',
 'PyYAML>=6.0,<7.0',
 'XlsxWriter>=3.0.3,<4.0.0',
 'json5>=0.9.8,<0.10.0',
 'pandas>=1.4.3,<2.0.0',
 'requests>=2.28.0,<3.0.0']

entry_points = \
{'console_scripts': ['opcli = opsrampcli.opsrampcli:main']}

setup_kwargs = {
    'name': 'opsrampcli',
    'version': '0.4.2',
    'description': '',
    'long_description': '# OpsRamp General Purpose Utility\n\nThis is a utility script that allows easy searching and manipulation of various OpsRamp items and configurations.  Before trying out any of the commands, you\'ll first need to [set up an environments file](#credentials) to contain the credentials you need to use.\n\n## Available Commands\nThe available commands are:\n\n\n| opcli Command            |Description                                                                                                                |\n|----------------------|---------------------------------------------------------------------------------------------------------------------------|\n| [getalerts](#getalerts)            |Search for and take action on alerts|\n| [postalerts](#postalerts)           |Post alerts to a Webhook integration (using vtoken) or directly to the API (using OAuth creds)|\n| [getincidents](#getincidents)        |Search and take action on Incidents|\n| [getresources](#getresources)        |Search for and take action on resources/devices|\n| [exportservicemaps](#exportservicemaps)   |Export one or more full Service Map definitions to a file which can be manipulated and re-imported|\n| [importservicemaps](#importservicemaps)   |Import (and optionally transform while doing so) from a Service Map export file|\n| [cloneservicemaps](#cloneservicemaps)    |Copy an existing Service Map, with transformations/replacements (useful when you have a template Service Map to re-use)|\n| [transformsvcmap](#transformsvcmap)     |Apply regex replacements to an exported Service Map and create a new transformed export file with the changes|\n| [getservicemaps](#getservicemaps)      |Get Service Map definitions|\n| [getchildsvcgroups](#getchildsvcgroups)   |Get child Service Groups of a parent Service|\n| [getservicegroup](#getservicegroup)     |Get the full definition of a Service Group|\n| [exportcustattrfile](#exportcustattrfile)  |Generate an Excel or csv file from existing custom attribute values|\n| [importcustattrfile](#importcustattrfile)  |Import an Excel file containing custom attribute values|\n| [getcustomattrs](#getcustomattrs)      |Get custom attribute definitions|\n| [getdiscoprofile](#getdiscoprofile)     |Get discovery profile definition|\n| [getalertesc](#getalertesc)         |Search and get Escalation Policy definitions|\n| [migratealertesc](#migratealertesc)     |Migrate/copy Escalation Policies within same tenant or from one tenant to another|\n\n\n&nbsp;\n&nbsp;\n\nFor more detailed help on a specific command use the -h option with the command, for example to get help on the cloneservicemaps command:\n\n```shell\n% opcli cloneservicemaps -h\nusage: opcli cloneservicemaps [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] --name NAME --replace REGEX REPLACEWITH [--parentlink] [--clobber]\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --env ENV             Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE       Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE     Location of environments YAML file (default: environments.yml)\n  --name NAME           Name of Service Map to transform and clone (default: None)\n  --replace REGEX REPLACEWITH\n                        Transforming regex pattern and replacement string (option can be repeated) (default: None)\n  --parentlink          If root Service has a link to a parent, link the imported Service Map (default: False)\n  --clobber             Overwrite Service Map (i.e. with same name) if it already exists (default: False)\n```\n\n## Credentials\n\nBy default, opcli requires an *environments.yml* file to be located in the directory where you running it.  Optionally you can specify an explicit environments file path & name using the --envfile option.  The file must follow the following format, where the name is the value you will specify to the --env option to define the URL and credentials you will use:\n\n```yaml\n- name: example1\n  url:  https://customer2name.api.opsramp.com\n  partner: msp_nnnnn\n  tenant: client_nnnnn\n  client_id: abcdefg1234567\n  client_secret: abcdefg1234567\n  vtoken: abcdefg1234567\n\n- name: example2\n  url:  https://customer2name.api.opsramp.com\n  partner: msp_nnnnn\n  tenant: client_nnnnn\n  client_id: abcdefg1234567\n  client_secret: abcdefg1234567\n  vtoken: abcdefg1234567\n```\nNote that you only need to provide the attributes needed for the kind of API calls you are making.  Therefore:\n* If you are making an API call that does not require a partner id (which is not needed  in most cases) you do not need to provide a *partner* value.\n* If you are not doing webhook alert posts, you would not need to specify the *vtoken* value.\n* If you are doing only webhook alert posts then you would not need *client_id* and *client_secret*.\n* If you are using the environment for a bunch of different scenarios, there is no harm in having the extra values there for the cases where they are not needed.\n\n&nbsp;\n&nbsp;\n# Commands\n\n## getalerts\nSearch for and take action on alerts\n\n```shell\nusage: opcli getalerts [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] --query QUERY [--brief] [--descr] [--count] [--filter FILTER] [--action ACTION] [--heal]\n\noptional arguments:\n  -h, --help         show this help message and exit\n  --env ENV          Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE    Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE  Location of environments YAML file (default: environments.yml)\n  --query QUERY      Query String to filter alerts as per https://develop.opsramp.com/resource-management/tenants-tenantid-resources-search (default: None)\n  --brief            Include only key fields in output (default: False)\n  --descr            Include the description field in results (runs *much* slower as it requires a separate api call per alert) (default: False)\n  --count            Only show the count of matching alerts (default: False)\n  --filter FILTER    Post-query filter on alerts. Python expression that will evaluate to True or False such as alert["resource"]["name"].startswith("prod") (default: None)\n  --action ACTION    Perform an action on matching alerts (Heal, acknowledge, suppress, close, unsuppress, unAcknowledge) (default: None)\n  --heal             Heal the matching alerts (i.e. send a matching Ok) (default: False)\n```\n### Alert search criteria\nAlert search --query option uses the syntax documented [here](https://web.archive.org/web/20211024221749/https://docs.opsramp.com/api/alerts/tenants-tenantid-alerts-search/)\n\n*Note: Above link is to an Internet Archive capture of an old doc page, as the latest doc no longer lists query variables* \n### Examples\n\nSearch for all alerts with matching resource/device numeric id and metric\n\n    % opcli getalerts --env myenvname --query \'resourceIds:9798408+metrics:testmetric\' --brief\n\n\nSearch for all alerts last updated during given time range, and heal & close them\n\n    % opcli getalerts --env myenvname --query \'startDate:2000-01-01T00:00:00 0000+endDate:2020-12-01T00:00:00 0000+alertTimeBase:updated+states:Critical,Warning,Info\' --heal --action close\n\n## postalerts\nPost alerts to a Webhook integration (using vtoken) or directly to the API (using OAuth creds)\n```shell\nusage: opcli postalerts [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] --infile INFILE [--range RANGE] [--auth AUTH]\n\noptional arguments:\n  -h, --help         show this help message and exit\n  --env ENV          Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE    Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE  Location of environments YAML file (default: environments.yml)\n  --infile INFILE    File containing an array of json alert payloads (default: None)\n  --range RANGE      An integer or range identifying which alert in the file to send (default: all)\n  --auth AUTH        Authentication method: vtoken or oauth (default: vtoken)\n```\n\n### Example\nPost alert #5 from the specified json file (expected to contain an array of alerts), using vtoken authentication\n\n    % opcli postalerts --env myenvname --infile myalertsamples.json --auth vtoken --range 5\n\n## getincidents\nSearch and take action on Incidents\n```shell\nusage: opcli getincidents [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] --query QUERY [--brief] [--count] [--filter FILTER] [--resolve]\n\noptional arguments:\n  -h, --help         show this help message and exit\n  --env ENV          Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE    Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE  Location of environments YAML file (default: environments.yml)\n  --query QUERY      Query String to filter incidents (default: None)\n  --brief            Include only key fields in output (default: False)\n  --count            Only show the count of matching incidents (default: False)\n  --filter FILTER    Post-query filter on incidents. Python expression that will evaluate to True or False such as incident["resource"]["name"].startswith("prod") (default: None)\n  --resolve          Resolve the matching incidents (default: False)\n```\n\n## getresources\nSearch for and take action on resources/devices\n```shell\nusage: opcli getresources [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] [--query QUERY] [--search SEARCH] [--count] [--delete] [--manage] [--filter FILTER]\n\noptional arguments:\n  -h, --help         show this help message and exit\n  --env ENV          Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE    Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE  Location of environments YAML file (default: environments.yml)\n  --query QUERY      Query String to filter resources as per https://develop.opsramp.com/resource-management/tenants-tenantid-resources-search (default: None)\n  --search SEARCH    Search String to filter resources as it would be entered under Resources -> Search (default: None)\n  --count            Only show the count of matching resources (default: False)\n  --delete           Delete the matching resources (default: False)\n  --manage           Manage the matching resources (default: False)\n  --filter FILTER    Post-query filter on resources. Python expression that will evaluate to True or False such as alert["resource"]["name"].startswith("prod") (default: None)\n```\n\n## exportservicemaps    \nExport one or more full Service Map definitions to a file which can be manipulated and re-imported|\n```shell\nusage: opcli exportservicemaps [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] [--name NAME] [--outdir OUTDIR] [--clobber] [--timestamp]\n\noptional arguments:\n  -h, --help         show this help message and exit\n  --env ENV          Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE    Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE  Location of environments YAML file (default: environments.yml)\n  --name NAME        Name of the root level Service Map/Group (export all if not specified) (default: None)\n  --outdir OUTDIR    Directory path where export will be saved (default: .)\n  --clobber          Remove/overwrite prior exports of same maps (default: False)\n  --timestamp        Include a timestamp in the Service Map dir name (default: False)\n```\n\n## importservicemaps    \nImport (and optionally transform while doing so) from a Service Map export file|\n```shell\nusage: opcli importservicemaps [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] --src SRC [--replace REGEX REPLACEWITH] [--parentlink] [--clobber]\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --env ENV             Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE       Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE     Location of environments YAML file (default: environments.yml)\n  --src SRC             Source: Path to the export file of a Service Map (default: None)\n  --replace REGEX REPLACEWITH\n                        Transforming regex pattern and replacement string (option can be repeated) (default: None)\n  --parentlink          If root Service has a link to a parent, link the imported Service Map (default: False)\n  --clobber             Overwrite Service Map (i.e. with same name) if it already exists (default: False)\n```\n\n## cloneservicemaps     \nCopy an existing Service Map, with transformations/replacements (useful when you have a template Service Map to re-use)|\n```shell\nusage: opcli cloneservicemaps [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] --name NAME --replace REGEX REPLACEWITH [--parentlink] [--clobber]\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --env ENV             Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE       Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE     Location of environments YAML file (default: environments.yml)\n  --name NAME           Name of Service Map to transform and clone (default: None)\n  --replace REGEX REPLACEWITH\n                        Transforming regex pattern and replacement string (option can be repeated) (default: None)\n  --parentlink          If root Service has a link to a parent, link the imported Service Map (default: False)\n  --clobber             Overwrite Service Map (i.e. with same name) if it already exists (default: False)\n```\n\n### Example\n\n```shell\n% opcli cloneservicemaps --env myenv --name "HUB" --replace HUB HUBCOPY --parentlink\n```\n## transformsvcmap      \nApply regex replacements to an exported Service Map and create a new transformed export file with the changes|\n```shell\nusage: opcli transformsvcmap [-h] src dest --replace REGEX REPLACEWITH [--clobber]\n\npositional arguments:\n  src                   Source: File path where a Service Map was previously exported\n  dest                  Destination: File path where the transformed export will be saved\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --replace REGEX REPLACEWITH\n                        Transforming regex pattern and replacement string (option can be repeated) (default: None)\n  --clobber             Overwrite dest file if it already exists (default: False)\n```\n\n## getservicemaps       \nGet Service Map definitions|\n```shell\nusage: opcli getservicemaps [-h] --env ENV [--secure SECURE] [--envfile ENVFILE]\n\noptional arguments:\n  -h, --help         show this help message and exit\n  --env ENV          Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE    Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE  Location of environments YAML file (default: environments.yml)\n```\n\n## getchildsvcgroups    \nGet child Service Groups of a parent Service|\n```shell\nusage: opcli getchildsvcgroups [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] --parent PARENT\n\noptional arguments:\n  -h, --help         show this help message and exit\n  --env ENV          Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE    Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE  Location of environments YAML file (default: environments.yml)\n  --parent PARENT    ID of the parent Service Map/Group (default: None)\n```\n\n## getservicegroup      \nGet the full definition of a Service Group|\n```shell\nusage: opcli getservicegroup [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] --id ID\n\noptional arguments:\n  -h, --help         show this help message and exit\n  --env ENV          Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE    Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE  Location of environments YAML file (default: environments.yml)\n  --id ID            ID of the Service Map/Group (default: None)\n```\n\n## exportcustattrfile   \nGenerate an Excel or csv file from existing custom attribute values|\n```shell\nusage: opcli exportcustattrfile [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] [--query QUERY] [--search SEARCH] [--filter FILTER] [--filename FILENAME]\n\noptional arguments:\n  -h, --help           show this help message and exit\n  --env ENV            Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE      Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE    Location of environments YAML file (default: environments.yml)\n  --query QUERY        Query String to filter resources as per https://develop.opsramp.com/resource-management/tenants-tenantid-resources-search (default: None)\n  --search SEARCH      Search String to filter resources as it would be entered under Resources -> Search (default: None)\n  --filter FILTER      Post-query filter on resources. Python expression that will evaluate to True or False such as alert["resource"]["name"].startswith("prod") (default: None)\n  --filename FILENAME  Name of excel file to generate (.xlsx extension will be added) (default: None)\n```\n\n## importcustattrfile   \nImport an Excel file containing custom attribute values|\n```shell\nusage: opcli importcustattrfile [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] [--commit] [--writeblanks] [--filename FILENAME]\n\noptional arguments:\n  -h, --help           show this help message and exit\n  --env ENV            Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE      Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE    Location of environments YAML file (default: environments.yml)\n  --commit             Make the actual updates on the platform. If not specified, only error checking and import simulation will occur. (default: False)\n  --writeblanks        When no value is provided in the spreadsheet for a resource, remove any existing value for that resource on the platform. If not specified then no action is taken for empty values. (default: False)\n  --filename FILENAME  Name of excel file to import (.xlsx extension will be added if not specified.) (default: None)\n```\n\n## getcustomattrs       \nGet custom attribute definitions|\n```shell\nusage: opcli getcustomattrs [-h] --env ENV [--secure SECURE] [--envfile ENVFILE]\n\noptional arguments:\n  -h, --help         show this help message and exit\n  --env ENV          Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE    Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE  Location of environments YAML file (default: environments.yml)\n```\n\n## getdiscoprofile      \nGet discovery profile definition|\n```shell\nusage: opcli getdiscoprofile [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] --id ID --tenantId TENANTID\n\noptional arguments:\n  -h, --help           show this help message and exit\n  --env ENV            Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE      Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE    Location of environments YAML file (default: environments.yml)\n  --id ID              Discovery profile ID (default: None)\n  --tenantId TENANTID  Client ID or MSP ID of the tenant (default: None)\n```\n\n## getalertesc          \nSearch and get Escalation Policy definitions|\n```shell\nusage: opcli getalertesc [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] [--query QUERY] [--details] [--count] [--filter FILTER]\n\noptional arguments:\n  -h, --help         show this help message and exit\n  --env ENV          Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE    Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE  Location of environments YAML file (default: environments.yml)\n  --query QUERY      Query String to filter alerts (default: None)\n  --details          Get the full details for all matched policies (default: False)\n  --count            Only show the count of matching alerts (default: False)\n  --filter FILTER    Post-query filter on alerts. Python expression that will evaluate to True or False such as alert["resource"]["name"].startswith("prod") (default: None)\n```\n\n## migratealertesc      \nMigrate/copy Escalation Policies within same tenant or from one tenant to another|\n```shell\nusage: opcli migratealertesc [-h] --env ENV [--secure SECURE] [--envfile ENVFILE] [--query QUERY] [--filter FILTER] [--preexec PREEXEC] [--postexec POSTEXEC] --to_env TO_ENV [--test] [--update] [--setactive SETACTIVE]\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --env ENV             Name of environment to use, as defined in your environments.yml file (default: None)\n  --secure SECURE       Whether or not to verify SSL cert (default: True)\n  --envfile ENVFILE     Location of environments YAML file (default: environments.yml)\n  --query QUERY         Query string to filter policies from source/from instance (default: None)\n  --filter FILTER       Filter for which policies to migrate. Python expression that will evaluate to True or False such as alert["resource"]["name"].startswith("prod") (default: None)\n  --preexec PREEXEC     Pre-mapped exec command (default: None)\n  --postexec POSTEXEC   Post-mapped exec command (default: None)\n  --to_env TO_ENV       Target environment to which policy definitions will be migrated. (--env option defines the source/from environment) (default: None)\n  --test                Test run only. Will check object mappings for missing items and not actually change the target instance. (default: False)\n  --update              Used for bulk updates, will only work if --env and --to_env are the same. Try to update existing policies instead of creating new ones. (default: False)\n  --setactive SETACTIVE\n                        Specify ON or OFF. Will force all policies created on the target to be ON or OFF. Otherwise will be set the same as the source. (default: None)\n```\n\n\n\n\n',
    'author': 'Michael Friedhoff',
    'author_email': 'michael.friedhoff@opsramp.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
